/* wydawca - FTP release synchronization daemon
   Copyright (C) 2007 Sergey Poznyakoff

   Wydawca is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 3 of the License, or (at your
   option) any later version.

   Wydawca is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with wydawca. If not, see <http://www.gnu.org/licenses/>. */

#include "wydawca.h"
#include <mail.h>

int mailer_opened;
mu_mailer_t mailer;
mu_address_t admin_address;
mu_address_t from_address;
unsigned long mail_admin_mask;
char *admin_stat_message_template;
unsigned long owner_notification_flags;
char *user_message_template[MAX_EVENT];

void
mail_init ()
{
  if (!mailer)
    {
      int rc;
      if ((rc = mu_mailer_create (&mailer, NULL)))
	{
	  const char *url = NULL;
	  mu_mailer_get_url_default (&url);
	  logmsg (LOG_ERR, "cannot create default mailer `%s': %s",
		  url, mu_strerror (rc));
	}
    }
}

void
mail_send_message (mu_address_t rcpt, const char *text)
{
  int rc;
  mu_message_t msg;
  mu_stream_t stream = NULL;
  mu_header_t hdr;
  int mailer_flags = 0;
  static char *x_mailer = "wydawca (" PACKAGE_STRING ")";
  size_t size;
  char *buf;

  mu_message_create (&msg, NULL);

  mu_message_get_stream (msg, &stream);
  mu_stream_write (stream, text, strlen (text), 0, NULL);
  mu_message_get_header (msg, &hdr);
  mu_header_append (hdr, "X-Mailer", x_mailer);

  mu_address_to_string (rcpt, NULL, 0, &size);
  buf = xmalloc (size + 1);
  mu_address_to_string (rcpt, buf, size + 1, NULL);

  mu_header_set_value (hdr, "To", buf, 1);
  free (buf);
  
  if (debug_level > 1)
    {
      mu_debug_t debug;
      mu_mailer_get_debug (mailer, &debug);
      mu_debug_set_level (debug, MU_DEBUG_TRACE | MU_DEBUG_PROT);
      if (debug_level > 2)
	mailer_flags = MAILER_FLAG_DEBUG_DATA;
    }

  if (!mailer_opened)
    {
      if ((rc = mu_mailer_open (mailer, mailer_flags)))
	{
	  const char *url = NULL;
	  mu_mailer_get_url_default (&url);
	  logmsg (LOG_CRIT, "opening mailer `%s' failed: %s",
		  url, mu_strerror (rc));
	  return;
	}
      mailer_opened = 1;
    }
  
  rc = mu_mailer_send_message (mailer, msg, from_address, rcpt);
  if (rc)
    logmsg (LOG_CRIT, "cannot send message: %s", mu_strerror (rc));

  mu_message_destroy (&msg, mu_message_get_owner (msg));
}

void
mail_stats ()
{
  struct kw_expansion exp[MAX_STAT + 1];
  time_t t;
  char *text;
  
  if (!stat_mask_p (mail_admin_mask) || !mailer)
    return;

  if (debug_level)
    {
      size_t size;
      char *buf;
      mu_address_to_string (admin_address, NULL, 0, &size);
      buf = xmalloc (size + 1);
      mu_address_to_string (admin_address, buf, size + 1, NULL);
      logmsg (LOG_DEBUG, "Sending stats to %s", buf);
      free (buf);
    }
  
  if (dry_run_mode)
    return;

  time (&t);
  exp[0].kw = "date";
  exp[0].value = xstrdup (ctime (&t));
  exp[0].value [strlen (exp[0].value) - 1] = 0;
  exp[0].expand = NULL;
  
  make_stat_expansion (exp + 1);

  text = expand_param (admin_stat_message_template,
		       exp, sizeof (exp) / sizeof (exp[0]), NULL);

  mail_send_message (admin_address, text);

  free (text);
  free_kwexp (exp, sizeof (exp) / sizeof (exp[0]));
}

void
mail_finish ()
{
  if (mailer_opened)
    mu_mailer_close (mailer);
}  

void
notify_owner (struct file_triplet *trp, enum notification_event ev)
{
  int rc;
  struct access_method *method;
  char *text;
  mu_address_t rcpt = NULL;
  unsigned nrows, ncols, i;
  struct kw_expansion kwexp[4];

  switch (ev)
    {
    case ev_success:
    case ev_bad_directive_signature:
    case ev_bad_detached_signature:
      method = trp->dpair->user_data_method;
      break;
      
    case ev_bad_ownership:
      method = trp->dpair->project_owner_method;
      break;

    default:
      abort ();
    }
  
  if (method->type == method_none)
    {
      logmsg (LOG_NOTICE,
	      "NOT notifying project admins: project-owner not configured in "
	      "[%s, %s]", trp->dpair->source_dir, trp->dpair->dest_dir);
      return;
    }

  if (method_init (method))
    {
      logmsg (LOG_ERR,
	      "failed to initialize project-owner method: "
	      "NOT notifying project admins");
      return;
    }

  make_default_kwexp (kwexp, trp->user, trp->project);
  escape_kwexp (method, kwexp, NITEMS (kwexp));
  text = expand_param (method->param[1], kwexp, NITEMS (kwexp), NULL);

  rc = method_run (method, text);
  free (text);
  if (rc)
    {
      logmsg (LOG_ERR, "cannot obtain owner emails for %s",
	      trp->project);
      return;
    }
  
  nrows = method_num_rows (method);
  ncols = method_num_cols (method);

  if (nrows == 0)
    {
      logmsg (LOG_ERR, "cannot obtain owner emails for %s",
	      trp->project);
      return;
    }
  
  for (i = 0; i < nrows; i++)
    {
      mu_address_t addr;
      const char *str = method_result (method, i, 0);
      if (mu_address_create (&addr, str))
	continue;
      if (ncols > 0)
	{
	  str = method_result (method, i, 1);
	  if (str)
	    mu_address_set_personal (addr, 1, str);
	}
      mu_address_union (&rcpt, addr);
      mu_address_destroy (&addr);
    }

  if (debug_level)
    {
      size_t size;
      char *buf;
      mu_address_to_string (rcpt, NULL, 0, &size);
      buf = xmalloc (size + 1);
      mu_address_to_string (rcpt, buf, size + 1, NULL);
      logmsg (LOG_DEBUG, "Notifying admins of %s: %s", trp->project,
	      buf);
      free (buf);
    }

  if (!dry_run_mode)
    {
      text = triplet_expand_param (user_message_template[ev], trp);
      mail_send_message (rcpt, text);
      free (text);
    }
  
  mu_address_destroy (&rcpt);
}

void
notify (struct file_triplet *trp, enum notification_event ev)
{
  fill_project_name (trp);
  if (owner_notification_flags & STAT_MASK (ev))
    notify_owner (trp, ev);
  /* FIXME */
}
